#include "gamestate.h"
#include "widgets/button.h"
#include "widgets/group.h"

#include "core/game.h"

GameState::GameState(sf::RenderWindow& game_window, const std::shared_ptr<Game>& game,
                     Callbacks&& callbacks) :
    _game(game),
    _game_window(game_window),
    _onLeaveGameCallback(callbacks.onLeaveGame)
{}

void GameState::input(const sf::Event& event)
{
    if (event.key.code == sf::Keyboard::Space && event.type == sf::Event::KeyReleased)
        _music.isPaused() ? _music.play() : _music.pause();

    _game->input(PlayerInput{_music.fetchOffset(), event});
}

void GameState::update(const sf::Time& dt)
{
    _game->update(UpdateData{_music.fetchOffset(), dt});
}

void GameState::draw() const
{
    _game->draw();
}

void GameState::enter()
{
    _game->run();

    _music.openFromFile("METEOR.flac");
    _music.setVolume(10);
    _music.play();
}

void GameState::leave()
{
    _onLeaveGameCallback();
}
