#pragma once

#include "widget.h"

class EditorWidget : public Widget
{
public:

    struct Callbacks
    {
        std::function<void(const sf::Event& event)> onInput;
        std::function<void(const sf::Time& dt)> onUpdate;
        std::function<void(sf::RenderTarget& target, sf::RenderStates states)> onDraw;
    };

    explicit EditorWidget(Callbacks&& callbacks);

    virtual void input(const sf::Event& event) override;
    virtual void update(const sf::Time& dt) override;
    virtual void draw(sf::RenderTarget& target, sf::RenderStates states) const override;
    virtual void move(const sf::Vector2f& delta) override;
    virtual bool isUnderMouse(int mouse_x, int mouse_y) const override;

    virtual void setRect(const sf::FloatRect& rect) override;
    virtual sf::FloatRect rect() const override;

    virtual void setPosition(const sf::Vector2f& position) override;
    virtual sf::Vector2f position() const override;

private:
    std::function<void(const sf::Event& event)> _input;
    std::function<void(const sf::Time& dt)> _update;
    std::function<void(sf::RenderTarget& target, sf::RenderStates states)> _draw;
};

